<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Services\MapPortaLoja as MapPortaLojaService;
use Teknisa\MAN\Services\ProdLoja as ProdLojaService;
use Teknisa\MAN\Utilities\Util as Util;
use \Exception as Exception;

class ImpressoraNFLoja extends Crud {
 
    // Data Source Names 
    protected $dataSourceName = 'impressoraNFLoja',
              $dataSourcePortas = 'mapportaloja';
              
    // Services
    protected $dataSourceManager,
              $mapPortaService,
              $prodLojaService,
              $util;
  
    public function __construct (Manager $dataSourceManager,  MapPortaLojaService $mapPortaService, ProdLojaService $prodLojaService, Util $util) {
        $this->util = $util;
        $this->dataSourceManager = $dataSourceManager;
        $this->prodLojaService = $prodLojaService;
        $this->mapPortaService = $mapPortaService;
    }
    
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows =  $this->util->getParamsAsArray($request);
        if(empty($rows[0]['CDPORTAIMPR'])){
            $rows[0]['CDPORTAIMPR'] = $rows[0]['DSIPIMPR'];
        }
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
           
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        
        $rows =  $this->util->getParamsAsArray($request);
        //  You cannot delete a Printer used by a Product
            
        try {    
            foreach ($rows as $row) {
                
                $result = $this->prodLojaService->findProductsByPrinter($row['CDFILIAL'], $row['CDLOJA'], $row['NRSEQIMPRLOJA'], $row['NRORG']);
                
                // I don't know why, but there's an array inside another array
                if($result[0]['PRODUCTS'] > 0) {
                    throw new Exception('Essa impressora não pode ser deletada, pois ainda está relacionada à um produto');    
                } else {
                    try {
                        $nrDeletedRows = 0;
                        // Deletes the ports
                        foreach ($rows as $row) {
                            // Searches for ports related to the Printer
                            $filterCriteria = new FilterCriteria ($this->dataSourcePortas);
                            $filterCriteria->addCondition("CDFILIAL", FilterCriteria::EQ, $row["CDFILIAL"]);
                            $filterCriteria->addCondition("CDLOJA", FilterCriteria::EQ, $row["CDLOJA"]);
                            $filterCriteria->addCondition("NRORG", FilterCriteria::EQ, $row["NRORG"]);
                            $filterCriteria->addCondition("CDPORTAIMPR", FilterCriteria::EQ, $row["CDPORTAIMPR"]);
                            
                            $result = $this->mapPortaService->find($filterCriteria, $this->dataSourceManager, $this->dataSourcePortas);
                            
                            
                            // Deletes the ports found in the result
                            if(count($result) === 1) {
                                
                                $filterCriteriaImp = new FilterCriteria ($this->dataSourceName);
                                $filterCriteriaImp->addCondition("CDFILIAL", FilterCriteria::EQ, $row["CDFILIAL"]);
                                $filterCriteriaImp->addCondition("CDLOJA", FilterCriteria::EQ, $row["CDLOJA"]);
                                $filterCriteriaImp->addCondition("NRORG", FilterCriteria::EQ, $row["NRORG"]);
                                $filterCriteriaImp->addCondition("CDPORTAIMPR", FilterCriteria::EQ, $row["CDPORTAIMPR"]);
                                
                                $resultImp = $this->mapPortaService->find($filterCriteriaImp, $this->dataSourceManager, $this->dataSourceName);
                               
                                if(!count($resultImp)>1){
                                    $dataSet2 = new DataSet ($this->dataSourcePortas, $result);
                                    $deletedRows = $this->dataSourceManager->delete($dataSet2);
                                    $nrDeletedRows = count($deletedRows);
                                }
                            } 
                        }
                        
                        // Delete the Printer
                        $dataSet = $request->getDataSet();
                        $dataSet->setDataSourceName($this->dataSourceName);
                        $deletedRows = $this->dataSourceManager->delete($dataSet);
                        $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
                        $nrDeletedRows += count($deletedRows);
                        
                    } catch (\Exception $e) {
                        $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
                    }
                }    
            }
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}