<?php
namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities\Query;
use Doctrine\DBAL\Connection;

class PDVService {
    
    protected $connection;
    protected $entityManager;
    
    public function __construct(EntityManager $entityManager, Connection $connection) {
        $this->entityManager = $entityManager;
        $this->connection = $connection;
    }
    
    public function exportarCaixas($params) {
        
        $paramsQuery = array(
	     	":CDFILIAL"  => $params['filialAtual'],
	     	":CDCAIXA"   => $params['caixaAtual'],
	     	":CDLOJA"    => $params['lojaAtual'],
	     	":NRORG"     => $params['NRORG']
 		);
 		$rowExport = $this->connection->fetchAll(self::SELECT_CAIXA, $paramsQuery)[0];
 		
 		for($i = 0; $i < count($params['filiCaixas']); $i++) {
 		    $filialToExport = $params['filiais'][$i];
 		    $caixaToExport = $params['caixas'][$i];
 		    $lojaToExport = $params['lojas'][$i];
            $rowExport[":OLD_CDFILIAL"] = $filialToExport;
 		    $rowExport[":OLD_CDCAIXA"] = $caixaToExport;
 		    $rowExport[":OLD_CDLOJA"] = $lojaToExport;
 		    $rowExport[":NRORG"] = $params['NRORG'];
            
 		    $this->connection->executeQuery(self::EXPORT_PARATRIZACAO, $rowExport);
 		}
    }
    
    public function paramsImportExport ($CDFILIAL, $CDCAIXA, $NRORG, $data, $rowName, $query) {
        $params = array(
	     	":CDFILIAL"  => $CDFILIAL,
	     	":CDCAIXA"   => $CDCAIXA,
	     	":NRORG"     => $NRORG
 		);
        $rowExport = $this->entityManager->getConnection()->fetchAll(Query::GET_PARAMS_EXPORT_IMPORT, $params);
        $rowExport = count($rowExport) > 0 ? preg_replace('/__/','_',$rowExport[0][$rowName]) : "";
        for ($i = 1 ; $i < count($data); $i++) {
            $pos = strpos($rowExport, $data[$i]);
            if ($pos === false) {
                $concatenar = $data[$i];
                $rowExport .= $data[$i] . "_";
            } else 
                $concatenar = "";
        }
        $conditions = array(
            ":".$rowName  => '_'.$rowExport,
            ":CDFILIAL"   => $CDFILIAL,
            ":CDCAIXA"    => $CDCAIXA,
            ":NRORG"      => $NRORG
        );
        $this->connection->executeQuery($query, $conditions);
    }    
    
    public function deleteParamsImportExport ($params, $data, $rowName) {
        
 		$rowExport = $this->entityManager->getConnection()->fetchAll(self::GET_PARAMS_EXPORT_IMPORT, $params);
        $rowExport = count($rowExport) > 0 ? $rowExport[0][$rowName] : "";
        
        for($i = 1 ; $i < count($data); $i++) {
           $rowExport = str_replace("_".$data[$i]['CDCAIXA'], "", $rowExport);
        }
        $texto = preg_replace('/__/','_',$rowExport);
        
        $params = array_merge($params, array(":".$rowName  => $texto));
        
        if($rowName == 'DSCAIXAVEN'){
            $this->connection->executeQuery(self::UPDATE_PARAMS_EXPORT, $params);    
        }else{
            $this->connection->executeQuery(self::UPDATE_PARAMS_IMPORT, $params);
        }
        
    }
    
    public function deleteModalidades ($NRORG, $CDFILIAL, $CDCAIXA) {
        $params = array(
	     	"CDFILIAL"  => $CDFILIAL,
	     	"CDCAIXA"   => $CDCAIXA,
	     	"NRORG"     => $NRORG
 		);
        $this->connection->executeQuery(Query::DELETE_VND_RELACAIXAMODALIDADE, $params);
    }

    public function resetCaixa ($NRORG, $CDFILIAL, $CDCAIXA, $CDLOJA, $DTULTATU, $NRORGULTATU, $CDOPERULTATU) {
        $params = array(
            ":DTULTATU"     => $DTULTATU,
            ":NRORGULTATU"  => $NRORGULTATU,
            ":CDOPERULTATU" => $CDOPERULTATU,
            ":CDFILIAL"     => $CDFILIAL,
	     	":CDCAIXA"      => $CDCAIXA,
            ":CDLOJA"       => $CDLOJA,
            ":NRORG"        => $NRORG
        ); 
        $this->connection->executeQuery(Query::UPDATE_VND_RELALMOXARIFADOESTRUT_RESET_CAIXA, $params);
        $this->connection->executeQuery(Query::UPDATE_VND_RELACAIXAMODALIDADE, $params);
        $this->connection->executeQuery(Query::UPDATE_VND_CONFIGINDIVIDUALPDV, $params);
        $this->connection->executeQuery(Query::UPDATE_CAIXA, $params);
    }

    public function getNextNumModalidade() {
        return $this->connection->fetchAll(Query::GET_MAXNRRELCAIXAMODALIDADE);
    }

    public function findCD ($NRORG, $CDFILIAL, $CDCAIXA) {
        $params = array(
            "CDFILIAL"     => $CDFILIAL,
	     	"CDCAIXA"      => $CDCAIXA,
            "NRORG"        => $NRORG
        );
        $cdcaixa = $this->connection->fetchAll(Query::GET_CDCAIXA, $params);
        return !isset($cdcaixa[0]['CDCAIXA']) ;
    }
    
    public function findIDATIVO ($NRORG, $CDFILIAL, $CDCAIXA, $CDLOJA) {
        $params = array(
            "CDFILIAL"     => $CDFILIAL,
	     	"CDCAIXA"      => $CDCAIXA,
            "CDLOJA"       => $CDLOJA,
            "NRORG"        => $NRORG
        );
        $cdcaixa = $this->connection->fetchAll(Query::GET_CDCAIXA, $params);
        if (count($cdcaixa) == 0) return false;
        else return $cdcaixa[0]['IDATIVO'];   
    }
    
    public function findCDCaixa ($NRORG, $CDFILIAL, $CDCAIXA, $CDLOJA) {
        $params = array(
            "CDFILIAL"     => $CDFILIAL,
	     	"CDCAIXA"      => $CDCAIXA,
            "NRORG"        => $NRORG,
            "CDLOJA"       => $CDLOJA
        );
        $cdcaixa = $this->connection->fetchAll(Query::GET_CDCAIXA, $params);
        return isset($cdcaixa[0]['CDCAIXA']) ? false : true;
    }
    
    public function relacionaVendedor($NRORG, $CDFILIAL, $CDCAIXA, $CDVENDEDOR) {
        $params = array(
	     	"CDCAIXA"    => $CDCAIXA,
	     	"CDFILIAL"   => $CDFILIAL,
            "CDVENDEDOR" => $CDVENDEDOR,
            "NRORG"      => $NRORG
        );
        $this->connection->executeQuery(self::RELACIONA_VENDEDOR, $params);
    }
    
    public function verificaVendedor($NRORG, $CDVENDEDOR) {
        $params = array(
            "CDVENDEDOR" => $CDVENDEDOR,
            "NRORG"      => $NRORG
        );
        $resultQuery = $this->connection->fetchAll(self::VERIFICA_VENDEDOR, $params);
        return $resultQuery[0]['CDCAIXA'];
    }
    
    public function desassociaVendedor($NRORG, $CDVENDEDOR) {
        $params = array(
            "CDVENDEDOR" => $CDVENDEDOR,
            "NRORG"      => $NRORG
        );
        $this->connection->executeQuery(self::DESASSOCIA_VENDEDOR, $params);
    }
    
    public function checkTipoDeCaixa ($row){
        $params = array(
            "CDFILIAL"     => $row['CDFILIAL'],
	     	"CDCAIXA"      => $row['CDCAIXA'],
            "NRORG"        => $row['NRORG'],
            "CDLOJA"       => $row['CDLOJA']
        );
        $total = $this->connection->fetchAll(self::CHECK_TIPO_CAIXA, $params);
        return $total[0]['TOTAL'];
    }
    
    public function checkDeliverCentraliza($row){
        $params = array(
            "CDFILIAL"     => $row['CDFILIAL'],
	     	"CDCAIXA"      => $row['CDCAIXA'],
            "CDLOJA"       => $row['CDLOJA']
        );
            return $this->connection->fetchAll(self::CHECK_DELIVER_CENTRAL, $params);
         
    }
    const CHECK_DELIVER_CENTRAL = "
         SELECT
           CDCAIXA
        FROM
           CAIXA
        WHERE
           (CDFILIAL = :CDFILIAL) AND
           (CDLOJA   = :CDLOJA)  AND
           (CDCAIXA <> :CDCAIXA ) AND   
           (IDSINCCAIXADLV = 'S')
    ";
    
    const CHECK_TIPO_CAIXA = "
        SELECT COUNT (CDCAIXACONT) AS TOTAL
        FROM CAIXA
        WHERE (CDFILIAL   = :CDFILIAL)
        AND (CDLOJA       = :CDLOJA)
        AND (IDCONFVENDA <> '2')
        AND CDCAIXACONT   = :CDCAIXA
        AND NRORG         = :NRORG
    ";
    
    const UPDATE_PARAMS_EXPORT = "
        UPDATE CAIXA
         SET
          DSCAIXAVEN = :DSCAIXAVEN
        WHERE
          CDFILIAL = :CDFILIAL AND
          CDCAIXA = :CDCAIXA AND
          NRORG = :NRORG
      ";
      
      const UPDATE_PARAMS_IMPORT = "
        UPDATE CAIXA
         SET
          DSCAIXATAB = :DSCAIXATAB
        WHERE
          CDFILIAL = :CDFILIAL AND
          CDCAIXA = :CDCAIXA AND
          NRORG = :NRORG
      ";
  
      const GET_PARAMS_EXPORT_IMPORT = "
        SELECT 
          DSCAIXAVEN, DSCAIXATAB
        FROM
          CAIXA
        WHERE
          CDFILIAL = :CDFILIAL AND
          CDCAIXA = :CDCAIXA AND
          NRORG  = :NRORG
      ";
  
      const RELACIONA_VENDEDOR = "
        UPDATE VENDEDOR
            SET
                CDFILIAL = :CDFILIAL,
                CDCAIXA = :CDCAIXA 
        WHERE
            CDVENDEDOR = :CDVENDEDOR AND
            NRORG      = :NRORG
      ";
  
      const VERIFICA_VENDEDOR = "
        SELECT
            CDCAIXA
        FROM
            VENDEDOR
        WHERE
            CDVENDEDOR = :CDVENDEDOR AND
            NRORG = :NRORG
      ";
  
      const DESASSOCIA_VENDEDOR = "
        UPDATE
            VENDEDOR
        SET
            CDCAIXA = NULL
        WHERE
            CDVENDEDOR = :CDVENDEDOR AND
            NRORG = :NRORG
      ";
      
      const EXPORT_PARATRIZACAO = "
    UPDATE
       CAIXA
    SET
       CDTECLADO        = :CDTECLADO,             IDSELLOJTOT      = :IDSELLOJTOT,        CDGRUPPROD       = :CDGRUPPROD,
       IDTIPOCAIXA      = :IDTIPOCAIXA,           CDTIPOOPER       = :CDTIPOOPER,         DSSTRINGCON      = :DSSTRINGCON,
       IDUTILTEF        = :IDUTILTEF,             IDUTILCHEQ       = :IDUTILCHEQ,         CDLOJATEF        = :CDLOJATEF,
       CDTERTEF         = :CDTERTEF,              DSENDIPSITEF     = :DSENDIPSITEF,       IDINFCLIECAI     = :IDINFCLIECAI,
       IDTPTELAVE       = :IDTPTELAVE,            NRCONFTELA       = :NRCONFTELA,         CDFILICONFTE     = :CDFILICONFTE,
       TPEXTFOTOPR      = :TPEXTFOTOPR,           IDVERIFCOTA      = :IDVERIFCOTA,        VRABERCAIX       = :VRABERCAIX,
       IDAUTOIMPTAB     = :IDAUTOIMPTAB,          IDCOLETOR        = :IDCOLETOR,          DSCAIXAVEN       = :DSCAIXAVEN,
       DSCAIXATAB       = :DSCAIXATAB,            IDEMIRECSANG     = :IDEMIRECSANG,       IDEMIRELPROD     = :IDEMIRELPROD,
       IDTRIBRELPRO     = :IDTRIBRELPRO,          IDIMPREDZ        = :IDIMPREDZ,          IDAUTOIMPVEN     = :IDAUTOIMPVEN,
       IDDIGCODPLM      = :IDDIGCODPLM,           CDPORTAIMPFI     = :CDPORTAIMPFI,       IDTPVENDIMPNF1   = :IDTPVENDIMPNF1,
       IDTPVENDIMPNF2   = :IDTPVENDIMPNF2,        IDGERADEBUG      = :IDGERADEBUG,        CDPORTABAL       = :CDPORTABAL,
       IDTIPOBAL        = :IDTIPOBAL,             NRVELOCBAL       = :NRVELOCBAL,         IDTIPOGAVETA     = :IDTIPOGAVETA,
       IDTIPODISP       = :IDTIPODISP,            CDPORTADISP      = :CDPORTADISP,        IDBLOQIDCART     = :IDBLOQIDCART,
       DSOBSCUPFIS      = :DSOBSCUPFIS,           IDTIPOPROX       = :IDTIPOPROX,         CDPORTAPROX      = :CDPORTAPROX,
       NRVELOCPROX      = :NRVELOCPROX,           IDINFVENBAL      = :IDINFVENBAL,        IDIMPBARRCUP     = :IDIMPBARRCUP,
       NMPADBARRCUP     = :NMPADBARRCUP,          IDIMPLEITURAX    = :IDIMPLEITURAX,      IDSOLNUMDEPFC    = :IDSOLNUMDEPFC,
       IDPERMCANCVND    = :IDPERMCANCVND,         IDSOLICITACPF    = :IDSOLICITACPF,      IDBAIXAPERIF     = :IDBAIXAPERIF,
       IDGRAVAMFD       = :IDGRAVAMFD,            IDINFREDZNCAD    = :IDINFREDZNCAD,      IDTIPOMICROTERM  = :IDTIPOMICROTERM,
       IDTIPINFCONS     = :IDTIPINFCONS,          IDPAISINTEGRA    = :IDPAISINTEGRA,      CDPREFINTGCOL    = :CDPREFINTGCOL,
       NRCUPINIINTGCOL  = :NRCUPINIINTGCOL,       NRCUPFININTGCOL  = :NRCUPFININTGCOL,    DSRESOLINTGCOL   = :DSRESOLINTGCOL,
       IDPERCOMVENCPDC  = :IDPERCOMVENCPDC,       IDIMPVENDCON     = :IDIMPVENDCON,       DSMENSCPFCUP     = :DSMENSCPFCUP,
       IDLCDBARBALATOL  = :IDLCDBARBALATOL,       NRPOSINICODBARR  = :NRPOSINICODBARR,    NRPOSFINCODBARR  = :NRPOSFINCODBARR,
       NRPOSINIQTCDBAR  = :NRPOSINIQTCDBAR,       NRPOSFINQTCDBAR  = :NRPOSFINQTCDBAR,    NRPOSINIVRCDBAR  = :NRPOSINIVRCDBAR,
       NRPOSFINVRCDBAR  = :NRPOSFINVRCDBAR,       IDIMPCOMPDEBCON  = :IDIMPCOMPDEBCON,    IDIMPCOMPCREDPE  = :IDIMPCOMPCREDPE,
       IDIMPCOMPCUPVEN  = :IDIMPCOMPCUPVEN,       IDGERAARQVNDRIO  = :IDGERAARQVNDRIO,    IDPERABERCOMCXA  = :IDPERABERCOMCXA,
       IDTPCONTRREPIQ   = :IDTPCONTRREPIQ ,       NRDIASVENDAMSDE  = :NRDIASVENDAMSDE,    IDFREQLIMPVENDA  = :IDFREQLIMPVENDA,
       NRDIASBXVNDMSDE  = :NRDIASBXVNDMSDE,       IDUTILBINADEL    = :IDUTILBINADEL,      IDPALFUTRABRCXA  = :IDPALFUTRABRCXA,
       IDSOLCONICNFIS   = :IDSOLCONICNFIS,        IDSOLDIGOBSDC    = :IDSOLDIGOBSDC,      IDSOLDIGCONS     = :IDSOLDIGCONS,
       IDSOLTPSANGRIACX = :IDSOLTPSANGRIACX,      IDGERALOGFOS     = :IDGERALOGFOS,       VRPEMAXREPIQVND  = :VRPEMAXREPIQVND,
       IDIMPCOMPVCP     = :IDIMPCOMPVCP,          DSGRANCONTRIB    = :DSGRANCONTRIB,      DSAUTORETENT     = :DSAUTORETENT,
       IDUTILFECHCEGO   = :IDUTILFECHCEGO,        IDSINCCAIXADLV   = :IDSINCCAIXADLV,     IDGERBKPMSDE     = :IDGERBKPMSDE,
       IDIMPPEDPROD     = :IDIMPPEDPROD,          IDTPVENDIMPNF3   = :IDTPVENDIMPNF3,     IDSENHACUP       = :IDSENHACUP,
       IDTPTEF          = :IDTPTEF,               CDPORTATEF       = :CDPORTATEF,         CDESTTEF         = :CDESTTEF
    WHERE
       CDFILIAL     = :OLD_CDFILIAL
       AND CDCAIXA  = :OLD_CDCAIXA
       AND CDLOJA   = :OLD_CDLOJA
       AND NRORG    = :NRORG
  ";
  
  const SELECT_CAIXA = "
  SELECT CDTECLADO,       IDSELLOJTOT,    CDGRUPPROD,         IDTIPOCAIXA,          CDTIPOOPER,           DSSTRINGCON,        IDUTILTEF,
   IDUTILCHEQ,       CDLOJATEF,           CDTERTEF,           DSENDIPSITEF,         IDINFCLIECAI,         IDTPTELAVE,         NRCONFTELA,           CDFILICONFTE,
   TPEXTFOTOPR,      IDVERIFCOTA,         VRABERCAIX,         IDAUTOIMPTAB,         IDCOLETOR,            DSCAIXAVEN,         DSCAIXATAB,           IDEMIRECSANG,
   IDEMIRELPROD,     IDTRIBRELPRO,        IDIMPREDZ,          IDAUTOIMPVEN,         IDDIGCODPLM,          CDPORTAIMPFI,       IDTPVENDIMPNF1,       IDTPVENDIMPNF2,
   IDGERADEBUG,      CDPORTABAL,          IDTIPOBAL,          NRVELOCBAL,           IDTIPOGAVETA,         IDTIPODISP,         CDPORTADISP,          IDBLOQIDCART,
   IDTIPOPROX,       CDPORTAPROX,         NRVELOCPROX,        IDINFVENBAL,          IDIMPBARRCUP,         NMPADBARRCUP,       IDIMPLEITURAX,        IDSOLNUMDEPFC,
   IDSOLICITACPF,    IDBAIXAPERIF,        IDGRAVAMFD,         IDINFREDZNCAD,        IDTIPOMICROTERM,      IDTIPINFCONS,       IDPAISINTEGRA,        CDPREFINTGCOL,
   NRCUPINIINTGCOL,  NRCUPFININTGCOL,     DSRESOLINTGCOL,     IDPERCOMVENCPDC,      IDIMPVENDCON,         DSMENSCPFCUP,       IDLCDBARBALATOL,      NRPOSINICODBARR,
   NRPOSFINCODBARR,  NRPOSINIQTCDBAR,     NRPOSFINQTCDBAR,    NRPOSINIVRCDBAR,      NRPOSFINVRCDBAR,      IDIMPCOMPDEBCON,    IDIMPCOMPCREDPE,      IDIMPCOMPCUPVEN,
   IDGERAARQVNDRIO,  IDPERABERCOMCXA,     IDTPCONTRREPIQ,     NRDIASVENDAMSDE,      IDFREQLIMPVENDA,      NRDIASBXVNDMSDE,    IDUTILBINADEL,        IDPALFUTRABRCXA,
   IDSOLCONICNFIS,   IDSOLDIGOBSDC,       IDSOLDIGCONS,       IDSOLTPSANGRIACX,     IDGERALOGFOS,         VRPEMAXREPIQVND,    IDIMPCOMPVCP,         DSGRANCONTRIB,
   IDUTILFECHCEGO,   IDSINCCAIXADLV,      IDGERBKPMSDE,       IDIMPPEDPROD,         IDTPVENDIMPNF3,       DSOBSCUPFIS,        IDPERMCANCVND,        DSAUTORETENT,
   IDSENHACUP,       IDTPTEF,             CDPORTATEF,         CDESTTEF
  FROM CAIXA
  WHERE CDFILIAL = :CDFILIAL
    AND CDCAIXA  = :CDCAIXA
    AND CDLOJA   = :CDLOJA
    AND NRORG    = :NRORG
  ";
} 