<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities\Query;
use Teknisa\MAN\Utilities;

class ProdLojaCadastroAutomatico {

	protected $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager, Utilities\Util $util)  {
		$this->entityManager = $entityManager;
		$this->util          = $util; 
	}
	
	public function getIntervaloProdutos ($parametros) {
	   $todosProdutos = $this->entityManager->getConnection()->fetchAll(self::GET_INTERVALO_PRODUTOS, $parametros);
	    
	   $arrayProdutos = [];
	   foreach($todosProdutos as $produto)
	        array_push($arrayProdutos, $produto['CDPRODUTO']);
	        
	   return $arrayProdutos;
	}
	
	public function getIntervaloTodosProdutos ($parametros) {
	    $todosProdutos = $this->entityManager->getConnection()->fetchAll(self::GET_INTERVALO_TODOS_PRODUTOS, $parametros);
	    
	    $arrayProdutos = [];
	    
	    foreach($todosProdutos as $produto)
	       array_push($arrayProdutos, $produto['CDPRODUTO']);
	    
	    return $arrayProdutos;
	}
	
	
	public function cadastroAutomaticoInsert ($params, $arrayProdutos) {

	    $produtosComAImpressora = $this->getProdutosComDeterminadaImpressora($params);
	    sort($arrayProdutos);
	    sort($produtosComAImpressora);
	    
	    $impressorasJaCadastradas = [];
	    
		foreach($arrayProdutos as $produto) {
		   if(!in_array($produto, $produtosComAImpressora)) {
    		   $params[':CDPRODUTO'] = $produto;
    		   $params[':NRPRODAMBIENTE'] = $this->util->newCode('PRODAMBIENTE'.$params[':CDFILIAL'].$params[':CDLOJA'].$params[':CDPRODUTO'], 10);
    	       $this->entityManager->getConnection()->executeQuery(self::INSERT_PRODAMBIENTE, $params);
		   } else {
		       array_push($impressorasJaCadastradas, $produto);
		   }
		}

		return true;
	}
	
	public function getProdutosComDeterminadaImpressora ($params) {
	    $todosProdutos = $this->entityManager->getConnection()->fetchAll(self::GET_PROD_DETERMINADA_IMPRESSORA, $params);
	    
	    $arrayProdutos = [];
	    
	    foreach($todosProdutos as $produto)
	       array_push($arrayProdutos, $produto['CDPRODUTO']);
	    
	    return $arrayProdutos;
	}
	
	public function beginTransaction () {
		$this->entityManager->getConnection()->beginTransaction();
	}

	public function commit() {
		$this->entityManager->getConnection()->commit();
	}

	public function rollBack() {
		$this->entityManager->getConnection()->rollBack();
	}
	
	const GET_INTERVALO_PRODUTOS = "
        SELECT DISTINCT P.CDPRODUTO
		  FROM PRODUTO P
          JOIN PRODLOJA PL
            ON P.CDPRODUTO = PL.CDPRODUTO 
		 WHERE P.CDARVPROD BETWEEN :PRODINICIAL AND :PRODFINAL
		   AND P.NRORG = :NRORG
	";
	
	const GET_INTERVALO_TODOS_PRODUTOS = "
        SELECT DISTINCT PT.CDPRODUTO
        FROM PRODLOJA PL
        	JOIN PRODUTO PT
        		ON PL.CDPRODUTO   = PT.CDPRODUTO
        		AND PL.NRORG      = PT.NRORG
        WHERE PL.CDFILIAL = :CDFILIAL
        	AND PL.CDLOJA     = :CDLOJA
        	AND PL.NRORG      = :NRORG
	";
	
	  const INSERT_PRODAMBIENTE = "
        INSERT INTO PRODAMBIENTE
            (CDFILIAL, CDLOJA, CDPRODUTO, NRPRODAMBIENTE, NRCONFTELA, CDAMBIENTE, NRSEQIMPRLOJA, NRORG, DTINIVIGENCIA)
        VALUES
            (:CDFILIAL, :CDLOJA, :CDPRODUTO, :NRPRODAMBIENTE, :NRCONFTELA, :CDAMBIENTE, :NRSEQIMPRLOJA, :NRORG, :DTINIVIGENCIA)
    ";
    
    const GET_PROD_DETERMINADA_IMPRESSORA = "
        SELECT DISTINCT PA.CDPRODUTO
        FROM PRODAMBIENTE PA
        	JOIN AMBIENTES AM
        		ON AM.CDFILIAL    = PA.CDFILIAL
        		AND AM.NRCONFTELA = PA.NRCONFTELA
        		AND AM.CDAMBIENTE = PA.CDAMBIENTE
        	JOIN IMPRLOJA IL
        		ON IL.CDFILIAL       = PA.CDFILIAL
        		AND IL.CDLOJA        = PA.CDLOJA
        		AND IL.NRSEQIMPRLOJA = PA.NRSEQIMPRLOJA
        	JOIN CONFTELA CT
        		ON CT.CDFILIAL    = PA.CDFILIAL
        		AND CT.CDLOJA     = PA.CDLOJA
        		AND CT.NRCONFTELA = PA.NRCONFTELA
            AND CT.DTINIVIGENCIA = PA.DTINIVIGENCIA
        WHERE PA.CDFILIAL = :CDFILIAL
        	AND PA.CDLOJA     = :CDLOJA
        	AND PA.NRSEQIMPRLOJA  = :NRSEQIMPRLOJA
        	AND PA.CDAMBIENTE = :CDAMBIENTE
        	AND PA.NRCONFTELA = :NRCONFTELA
          AND PA.DTINIVIGENCIA = :DTINIVIGENCIA
    ";
}